@@ -1,11 +1,11 @@
 /* GCC internal format strings.
-   Copyright (C) 2003-2006 Free Software Foundation, Inc.
+   Copyright (C) 2003-2007 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -119,11 +118,16 @@ enum format_arg_type
   /* Bitmasks */
   FAT_SIZE_MASK		= (FAT_SIZE_LONG | FAT_SIZE_LONGLONG | FAT_SIZE_WIDE)
 };
+#ifdef __cplusplus
+typedef int format_arg_type_t;
+#else
+typedef enum format_arg_type format_arg_type_t;
+#endif
 
 struct numbered_arg
 {
   unsigned int number;
-  enum format_arg_type type;
+  format_arg_type_t type;
 };
 
 struct spec
@@ -152,8 +156,10 @@ numbered_arg_compare (const void *p1, const void *p2)
 }
 
 static void *
-format_parse (const char *format, bool translated, char **invalid_reason)
+format_parse (const char *format, bool translated, char *fdi,
+	      char **invalid_reason)
 {
+  const char *const format_start = format;
   struct spec spec;
   unsigned int unnumbered_arg_count;
   struct spec *result;
@@ -169,6 +175,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
     if (*format++ == '%')
       {
 	/* A directive.  */
+	FDI_SET (format - 1, FMTDIR_START);
 	spec.directives++;
 
 	if (*format == '%' || *format == '<' || *format == '>'
@@ -184,8 +191,8 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	    unsigned int flag_w = 0;
 	    unsigned int flag_plus = 0;
 	    unsigned int flag_sharp = 0;
-	    enum format_arg_type size;
-	    enum format_arg_type type;
+	    format_arg_type_t size;
+	    format_arg_type_t type;
 
 	    if (isdigit (*format))
 	      {
@@ -204,6 +211,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 		    if (m == 0)
 		      {
 			*invalid_reason = INVALID_ARGNO_0 (spec.directives);
+			FDI_SET (f, FMTDIR_ERROR);
 			goto bad_format;
 		      }
 		    number = m;
@@ -243,6 +251,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 		    continue;
 		  invalid_flags:
 		    *invalid_reason = xasprintf (_("In the directive number %u, the flags combination is invalid."), spec.directives);
+		    FDI_SET (format, FMTDIR_ERROR);
 		    goto bad_format;
 		  default:
 		    break;
@@ -270,10 +279,17 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 
 		    if (*format != 's')
 		      {
-			*invalid_reason =
-			  (*format == '\0'
-			   ? INVALID_UNTERMINATED_DIRECTIVE ()
-			   : xasprintf (_("In the directive number %u, a precision is not allowed before '%c'."), spec.directives, *format));
+			if (*format == '\0')
+			  {
+			    *invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+			    FDI_SET (format - 1, FMTDIR_ERROR);
+			  }
+			else
+			  {
+			    *invalid_reason =
+			      xasprintf (_("In the directive number %u, a precision is not allowed before '%c'."), spec.directives, *format);
+			    FDI_SET (format, FMTDIR_ERROR);
+			  }
 			goto bad_format;
 		      }
 
@@ -302,16 +318,19 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 			    if (m == 0)
 			      {
 				*invalid_reason = INVALID_WIDTH_ARGNO_0 (spec.directives);
+				FDI_SET (f, FMTDIR_ERROR);
 				goto bad_format;
 			      }
 			    if (unnumbered_arg_count > 0 || number == 0)
 			      {
 				*invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+				FDI_SET (f, FMTDIR_ERROR);
 				goto bad_format;
 			      }
 			    if (m != number - 1)
 			      {
 				*invalid_reason = xasprintf (_("In the directive number %u, the argument number for the precision must be equal to %u."), spec.directives, number - 1);
+				FDI_SET (f, FMTDIR_ERROR);
 				goto bad_format;
 			      }
 			    precision_number = m;
@@ -327,6 +346,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 			if (unnumbered_arg_count > 0)
 			  {
 			    *invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+			    FDI_SET (format - 1, FMTDIR_ERROR);
 			    goto bad_format;
 			  }
 
@@ -347,6 +367,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 			if (spec.numbered_arg_count > 0)
 			  {
 			    *invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+			    FDI_SET (format - 1, FMTDIR_ERROR);
 			    goto bad_format;
 			  }
 
@@ -364,16 +385,25 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 		      type = FAT_STRING;
 		    else
 		      {
-			*invalid_reason =
-			  (*format == '\0'
-			   ? INVALID_UNTERMINATED_DIRECTIVE ()
-			   : xasprintf (_("In the directive number %u, a precision is not allowed before '%c'."), spec.directives, *format));
+			if (*format == '\0')
+			  {
+			    *invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+			    FDI_SET (format - 1, FMTDIR_ERROR);
+			  }
+			else
+			  {
+			    *invalid_reason =
+			      xasprintf (_("In the directive number %u, a precision specification is not allowed before '%c'."), spec.directives, *format);
+			    FDI_SET (format, FMTDIR_ERROR);
+			  }
 			goto bad_format;
 		      }
 		  }
 		else
 		  {
 		    *invalid_reason = xasprintf (_("In the directive number %u, the precision specification is invalid."), spec.directives);
+		    FDI_SET (*format == '\0' ? format - 1 : format,
+			     FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 	      }
@@ -413,17 +443,24 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 		  type = FAT_TREE | FAT_TREE_CV;
 		else
 		  {
-		    *invalid_reason =
-		      (*format == '\0'
-		       ? INVALID_UNTERMINATED_DIRECTIVE ()
-		       : (*format == 'c'
-			  || *format == 's'
-			  || *format == 'i' || *format == 'd'
-			  || *format == 'o' || *format == 'u' || *format == 'x'
-			  || *format == 'H'
-			  ? xasprintf (_("In the directive number %u, flags are not allowed before '%c'."), spec.directives, *format)
-			  : INVALID_CONVERSION_SPECIFIER (spec.directives,
-							  *format)));
+		    if (*format == '\0')
+		      {
+			*invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+			FDI_SET (format - 1, FMTDIR_ERROR);
+		      }
+		    else
+		      {
+			*invalid_reason =
+			  (*format == 'c'
+			   || *format == 's'
+			   || *format == 'i' || *format == 'd'
+			   || *format == 'o' || *format == 'u' || *format == 'x'
+			   || *format == 'H'
+			   ? xasprintf (_("In the directive number %u, flags are not allowed before '%c'."), spec.directives, *format)
+			   : INVALID_CONVERSION_SPECIFIER (spec.directives,
+							   *format));
+			FDI_SET (format, FMTDIR_ERROR);
+		      }
 		    goto bad_format;
 		  }
 	      }
@@ -436,6 +473,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 		if (unnumbered_arg_count > 0)
 		  {
 		    *invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+		    FDI_SET (format, FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 
@@ -456,6 +494,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 		if (spec.numbered_arg_count > 0)
 		  {
 		    *invalid_reason = INVALID_MIXES_NUMBERED_UNNUMBERED ();
+		    FDI_SET (format, FMTDIR_ERROR);
 		    goto bad_format;
 		  }
 
@@ -470,6 +509,8 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	      }
 	  }
 
+	FDI_SET (format, FMTDIR_END);
+
 	format++;
       }
 
@@ -490,9 +531,9 @@ format_parse (const char *format, bool translated, char **invalid_reason)
       for (i = j = 0; i < spec.numbered_arg_count; i++)
 	if (j > 0 && spec.numbered[i].number == spec.numbered[j-1].number)
 	  {
-	    enum format_arg_type type1 = spec.numbered[i].type;
-	    enum format_arg_type type2 = spec.numbered[j-1].type;
-	    enum format_arg_type type_both;
+	    format_arg_type_t type1 = spec.numbered[i].type;
+	    format_arg_type_t type2 = spec.numbered[j-1].type;
+	    format_arg_type_t type_both;
 
 	    if (type1 == type2)
 	      type_both = type1;
@@ -523,7 +564,7 @@ format_parse (const char *format, bool translated, char **invalid_reason)
 	goto bad_format;
     }
 
-  result = (struct spec *) xmalloc (sizeof (struct spec));
+  result = XMALLOC (struct spec);
   *result = spec;
   return result;
 
@@ -656,7 +697,6 @@ struct formatstring_parser formatstring_gcc_internal =
    format_parse for strings read from standard input.  */
 
 #include <stdio.h>
-#include "getline.h"
 
 static void
 format_print (void *descr)
@@ -779,7 +819,7 @@ main ()
 	line[--line_len] = '\0';
 
       invalid_reason = NULL;
-      descr = format_parse (line, false, &invalid_reason);
+      descr = format_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -796,7 +836,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../lib -I../intl -DHAVE_CONFIG_H -DTEST format-gcc-internal.c ../lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../intl -DHAVE_CONFIG_H -DTEST format-gcc-internal.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
