@@ -1,13 +1,13 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995-1999, 2000-2006 Free Software Foundation, Inc.
+   Copyright (C) 1995-1999, 2000-2007 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>.
    Multibyte character handling by Bruno Haible <haible@clisp.cons.org>.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
 #ifdef HAVE_CONFIG_H
@@ -39,11 +38,10 @@
 
 #include "c-ctype.h"
 #include "linebreak.h"
-#include "vasprintf.h"
+#include "uniwidth.h"
 #include "gettext.h"
 #include "po-charset.h"
 #include "xalloc.h"
-#include "exit.h"
 #include "error.h"
 #include "error-progname.h"
 #include "xvasprintf.h"
@@ -57,7 +55,7 @@
 #define _(str) gettext(str)
 
 #if HAVE_ICONV
-# include "utf8-ucs4.h"
+# include "unistr.h"
 #endif
 
 #if HAVE_DECL_GETC_UNLOCKED
@@ -326,13 +324,13 @@ mb_setascii (mbchar_t mbc, char sc)
 
 /* Copying a character.  */
 static inline void
-mb_copy (mbchar_t new, const mbchar_t old)
+mb_copy (mbchar_t new_mbc, const mbchar_t old_mbc)
 {
-  memcpy_small (&new->buf[0], &old->buf[0], old->bytes);
-  new->bytes = old->bytes;
+  memcpy_small (&new_mbc->buf[0], &old_mbc->buf[0], old_mbc->bytes);
+  new_mbc->bytes = old_mbc->bytes;
 #if HAVE_ICONV
-  if ((new->uc_valid = old->uc_valid))
-    new->uc = old->uc;
+  if ((new_mbc->uc_valid = old_mbc->uc_valid))
+    new_mbc->uc = old_mbc->uc;
 #endif
 }
 
