@@ -1,11 +1,11 @@
 /* Writing binary .mo files.
-   Copyright (C) 1995-1998, 2000-2004 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2007 Free Software Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -38,15 +37,16 @@
 #include "gmo.h"
 #include "hash-string.h"
 
+#include "byteswap.h"
 #include "error.h"
 #include "hash.h"
 #include "message.h"
 #include "format.h"
+#include "xsize.h"
 #include "xalloc.h"
-#include "xallocsa.h"
+#include "xmalloca.h"
 #include "binary-io.h"
 #include "fwriteerror.h"
-#include "exit.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -67,10 +67,17 @@
 /* Alignment of strings in resulting .mo file.  */
 size_t alignment;
 
+/* True if writing a .mo file in opposite endianness than the host.  */
+bool byteswap;
+
 /* True if no hash table in .mo is wanted.  */
 bool no_hash_table;
 
 
+/* Destructively changes the byte order of a 32-bit value in memory.  */
+#define BSWAP32(x) (x) = bswap_32 (x)
+
+
 /* Indices into the strings contained in 'struct pre_message' and
    'struct pre_sysdep_message'.  */
 enum
@@ -137,6 +144,7 @@ struct pre_sysdep_message
 static void
 write_table (FILE *output_file, message_list_ty *mlp)
 {
+  char **msgctid_arr;
   size_t nstrings;
   struct pre_message *msg_arr;
   size_t n_sysdep_strings;
@@ -161,23 +169,33 @@ write_table (FILE *output_file, message_list_ty *mlp)
   /* First pass: Move the static string pairs into an array, for sorting,
      and at the same time, compute the segments of the system dependent
      strings.  */
+  msgctid_arr = XNMALLOC (mlp->nitems, char *);
   nstrings = 0;
-  msg_arr =
-    (struct pre_message *)
-    xmalloc (mlp->nitems * sizeof (struct pre_message));
+  msg_arr = XNMALLOC (mlp->nitems, struct pre_message);
   n_sysdep_strings = 0;
-  sysdep_msg_arr =
-    (struct pre_sysdep_message *)
-    xmalloc (mlp->nitems * sizeof (struct pre_sysdep_message));
+  sysdep_msg_arr = XNMALLOC (mlp->nitems, struct pre_sysdep_message);
   n_sysdep_segments = 0;
   sysdep_segments = NULL;
   have_outdigits = false;
   for (j = 0; j < mlp->nitems; j++)
     {
       message_ty *mp = mlp->item[j];
+      size_t msgctlen;
+      char *msgctid;
       struct interval *intervals[2];
       size_t nintervals[2];
 
+      /* Concatenate mp->msgctxt and mp->msgid into msgctid.  */
+      msgctlen = (mp->msgctxt != NULL ? strlen (mp->msgctxt) + 1 : 0);
+      msgctid = XNMALLOC (msgctlen + strlen (mp->msgid) + 1, char);
+      if (mp->msgctxt != NULL)
+	{
+	  memcpy (msgctid, mp->msgctxt, msgctlen - 1);
+	  msgctid[msgctlen - 1] = MSGCTXT_SEPARATOR;
+	}
+      strcpy (msgctid + msgctlen, mp->msgid);
+      msgctid_arr[j] = msgctid;
+
       intervals[M_ID] = NULL;
       nintervals[M_ID] = 0;
       intervals[M_STR] = NULL;
@@ -196,6 +214,22 @@ write_table (FILE *output_file, message_list_ty *mlp)
 
 	  get_sysdep_c_format_directives (mp->msgid, false,
 					  &intervals[M_ID], &nintervals[M_ID]);
+	  if (msgctlen > 0)
+	    {
+	      struct interval *id_intervals = intervals[M_ID];
+	      size_t id_nintervals = nintervals[M_ID];
+
+	      if (id_nintervals > 0)
+		{
+		  unsigned int i;
+
+		  for (i = 0; i < id_nintervals; i++)
+		    {
+		      id_intervals[i].startpos += msgctlen;
+		      id_intervals[i].endpos += msgctlen;
+		    }
+		}
+	    }
 
 	  p_end = mp->msgstr + mp->msgstr_len;
 	  for (p = mp->msgstr; p < p_end; p += strlen (p) + 1)
@@ -235,8 +269,9 @@ write_table (FILE *output_file, message_list_ty *mlp)
 	    {
 	      struct pre_sysdep_string *pre =
 		(struct pre_sysdep_string *)
-		xmalloc (sizeof (struct pre_sysdep_string)
-			 + nintervals[m] * sizeof (struct pre_segment_pair));
+		xmalloc (xsum (sizeof (struct pre_sysdep_string),
+			       xtimes (nintervals[m],
+				       sizeof (struct pre_segment_pair))));
 	      const char *str;
 	      size_t str_len;
 	      size_t lastpos;
@@ -244,8 +279,8 @@ write_table (FILE *output_file, message_list_ty *mlp)
 
 	      if (m == M_ID)
 		{
-		  str = mp->msgid;
-		  str_len = strlen (mp->msgid) + 1;
+		  str = msgctid; /* concatenation of mp->msgctxt + mp->msgid  */
+		  str_len = strlen (msgctid) + 1;
 		}
 	      else
 		{
@@ -313,8 +348,8 @@ write_table (FILE *output_file, message_list_ty *mlp)
       else
 	{
 	  /* Static string pair.  */
-	  msg_arr[nstrings].str[M_ID].pointer = mp->msgid;
-	  msg_arr[nstrings].str[M_ID].length = strlen (mp->msgid) + 1;
+	  msg_arr[nstrings].str[M_ID].pointer = msgctid;
+	  msg_arr[nstrings].str[M_ID].length = strlen (msgctid) + 1;
 	  msg_arr[nstrings].str[M_STR].pointer = mp->msgstr;
 	  msg_arr[nstrings].str[M_STR].length = mp->msgstr_len;
 	  msg_arr[nstrings].id_plural = mp->msgid_plural;
@@ -397,14 +432,12 @@ write_table (FILE *output_file, message_list_ty *mlp)
   /* Offset of table for original string offsets.  */
   header.orig_tab_offset = offset;
   offset += nstrings * sizeof (struct string_desc);
-  orig_tab =
-    (struct string_desc *) xmalloc (nstrings * sizeof (struct string_desc));
+  orig_tab = XNMALLOC (nstrings, struct string_desc);
 
   /* Offset of table for translated string offsets.  */
   header.trans_tab_offset = offset;
   offset += nstrings * sizeof (struct string_desc);
-  trans_tab =
-    (struct string_desc *) xmalloc (nstrings * sizeof (struct string_desc));
+  trans_tab = XNMALLOC (nstrings, struct string_desc);
 
   /* Size of hash table.  */
   header.hash_tab_size = hash_tab_size;
@@ -447,6 +480,24 @@ write_table (FILE *output_file, message_list_ty *mlp)
      compute the offsets of each string, including the proper alignment.  */
 
   /* Write the header out.  */
+  if (byteswap)
+    {
+      BSWAP32 (header.magic);
+      BSWAP32 (header.revision);
+      BSWAP32 (header.nstrings);
+      BSWAP32 (header.orig_tab_offset);
+      BSWAP32 (header.trans_tab_offset);
+      BSWAP32 (header.hash_tab_size);
+      BSWAP32 (header.hash_tab_offset);
+      if (minor_revision >= 1)
+	{
+	  BSWAP32 (header.n_sysdep_segments);
+	  BSWAP32 (header.sysdep_segments_offset);
+	  BSWAP32 (header.n_sysdep_strings);
+	  BSWAP32 (header.orig_sysdep_tab_offset);
+	  BSWAP32 (header.trans_sysdep_tab_offset);
+	}
+    }
   fwrite (&header, header_size, 1, output_file);
 
   /* Table for original string offsets.  */
@@ -462,6 +513,12 @@ write_table (FILE *output_file, message_list_ty *mlp)
       /* Subtract 1 because of the terminating NUL.  */
       orig_tab[j].length--;
     }
+  if (byteswap)
+    for (j = 0; j < nstrings; j++)
+      {
+	BSWAP32 (orig_tab[j].length);
+	BSWAP32 (orig_tab[j].offset);
+      }
   fwrite (orig_tab, nstrings * sizeof (struct string_desc), 1, output_file);
 
   /* Table for translated string offsets.  */
@@ -476,6 +533,12 @@ write_table (FILE *output_file, message_list_ty *mlp)
       /* Subtract 1 because of the terminating NUL.  */
       trans_tab[j].length--;
     }
+  if (byteswap)
+    for (j = 0; j < nstrings; j++)
+      {
+	BSWAP32 (trans_tab[j].length);
+	BSWAP32 (trans_tab[j].offset);
+      }
   fwrite (trans_tab, nstrings * sizeof (struct string_desc), 1, output_file);
 
   /* Skip this part when no hash table is needed.  */
@@ -487,7 +550,7 @@ write_table (FILE *output_file, message_list_ty *mlp)
       /* Here output_file is at position header.hash_tab_offset.  */
 
       /* Allocate room for the hashing table to be written out.  */
-      hash_tab = (nls_uint32 *) xmalloc (hash_tab_size * sizeof (nls_uint32));
+      hash_tab = XNMALLOC (hash_tab_size, nls_uint32);
       memset (hash_tab, '\0', hash_tab_size * sizeof (nls_uint32));
 
       /* Insert all value in the hash table, following the algorithm described
@@ -514,6 +577,9 @@ write_table (FILE *output_file, message_list_ty *mlp)
 	}
 
       /* Write the hash table out.  */
+      if (byteswap)
+	for (j = 0; j < hash_tab_size; j++)
+	  BSWAP32 (hash_tab[j]);
       fwrite (hash_tab, hash_tab_size * sizeof (nls_uint32), 1, output_file);
 
       free (hash_tab);
@@ -529,8 +595,7 @@ write_table (FILE *output_file, message_list_ty *mlp)
       /* Here output_file is at position header.sysdep_segments_offset.  */
 
       sysdep_segments_tab =
-	(struct sysdep_segment *)
-	xmalloc (n_sysdep_segments * sizeof (struct sysdep_segment));
+	XNMALLOC (n_sysdep_segments, struct sysdep_segment);
       for (i = 0; i < n_sysdep_segments; i++)
 	{
 	  offset = roundup (offset, alignment);
@@ -540,14 +605,19 @@ write_table (FILE *output_file, message_list_ty *mlp)
 	  offset += sysdep_segments_tab[i].length;
 	}
 
+      if (byteswap)
+	for (i = 0; i < n_sysdep_segments; i++)
+	  {
+	    BSWAP32 (sysdep_segments_tab[i].length);
+	    BSWAP32 (sysdep_segments_tab[i].offset);
+	  }
       fwrite (sysdep_segments_tab,
 	      n_sysdep_segments * sizeof (struct sysdep_segment), 1,
 	      output_file);
 
       free (sysdep_segments_tab);
 
-      sysdep_tab =
-	(nls_uint32 *) xmalloc (n_sysdep_strings * sizeof (nls_uint32));
+      sysdep_tab = XNMALLOC (n_sysdep_strings, nls_uint32);
       stoffset = sysdep_tab_offset;
 
       for (m = 0; m < 2; m++)
@@ -564,6 +634,9 @@ write_table (FILE *output_file, message_list_ty *mlp)
 			    * sizeof (struct segment_pair);
 	    }
 	  /* Write the table for original/translated sysdep string offsets.  */
+	  if (byteswap)
+	    for (j = 0; j < n_sysdep_strings; j++)
+	      BSWAP32 (sysdep_tab[j]);
 	  fwrite (sysdep_tab, n_sysdep_strings * sizeof (nls_uint32), 1,
 		  output_file);
 	}
@@ -579,7 +652,7 @@ write_table (FILE *output_file, message_list_ty *mlp)
 	    struct pre_sysdep_string *pre = msg->str[m];
 	    struct sysdep_string *str =
 	      (struct sysdep_string *)
-	      xallocsa (sizeof (struct sysdep_string)
+	      xmalloca (sizeof (struct sysdep_string)
 			+ pre->segmentcount * sizeof (struct segment_pair));
 	    unsigned int i;
 
@@ -596,12 +669,21 @@ write_table (FILE *output_file, message_list_ty *mlp)
 		str->segments[pre->segmentcount].segsize += msg->id_plural_len;
 		offset += msg->id_plural_len;
 	      }
+	    if (byteswap)
+	      {
+		BSWAP32 (str->offset);
+		for (i = 0; i <= pre->segmentcount; i++)
+		  {
+		    BSWAP32 (str->segments[i].segsize);
+		    BSWAP32 (str->segments[i].sysdepref);
+		  }
+	      }
 	    fwrite (str,
 		    sizeof (struct sysdep_string)
 		    + pre->segmentcount * sizeof (struct segment_pair),
 		    1, output_file);
 
-	    freesa (str);
+	    freea (str);
 	  }
     }
 
@@ -616,7 +698,7 @@ write_table (FILE *output_file, message_list_ty *mlp)
   offset = end_offset;
 
   /* A few zero bytes for padding.  */
-  null = alloca (alignment);
+  null = (char *) alloca (alignment);
   memset (null, '\0', alignment);
 
   /* Now write the original strings.  */
@@ -686,8 +768,11 @@ write_table (FILE *output_file, message_list_ty *mlp)
     }
 
   freea (null);
+  for (j = 0; j < mlp->nitems; j++)
+    free (msgctid_arr[j]);
   free (sysdep_msg_arr);
   free (msg_arr);
+  free (msgctid_arr);
 }
 
 
@@ -725,9 +810,6 @@ msgdomain_write_mo (message_list_ty *mlp,
 	  if (fwriteerror (output_file))
 	    error (EXIT_FAILURE, errno, _("error while writing \"%s\" file"),
 		   file_name);
-
-	  if (output_file != stdout)
-	    fclose (output_file);
 	}
     }
 
