@@ -1,12 +1,12 @@
 /* xgettext librep backend.
-   Copyright (C) 2001-2003, 2005-2006 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2005-2007 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,13 +14,15 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
+/* Specification.  */
+#include "x-librep.h"
+
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -33,7 +35,6 @@
 #include "x-librep.h"
 #include "error.h"
 #include "xalloc.h"
-#include "exit.h"
 #include "hash.h"
 #include "gettext.h"
 
@@ -179,7 +180,7 @@ static inline void
 init_token (struct token *tp)
 {
   tp->allocated = 10;
-  tp->chars = (char *) xmalloc (tp->allocated * sizeof (char));
+  tp->chars = XNMALLOC (tp->allocated, char);
   tp->charcount = 0;
 }
 
@@ -501,7 +502,7 @@ string_of_object (const struct object *op)
   if (!(op->type == t_symbol || op->type == t_string))
     abort ();
   n = op->token->charcount;
-  str = (char *) xmalloc (n + 1);
+  str = XNMALLOC (n + 1, char);
   memcpy (str, op->token->chars, n);
   str[n] = '\0';
   return str;
@@ -683,7 +684,7 @@ read_object (struct object *op, flag_context_ty outer_context)
 		else
 		  {
 		    /* These are the argument positions.  */
-		    if (inner.type == t_string)
+		    if (argparser != NULL && inner.type == t_string)
 		      arglist_parser_remember (argparser, arg,
 					       string_of_object (&inner),
 					       inner_context,
@@ -789,7 +790,7 @@ read_object (struct object *op, flag_context_ty outer_context)
 
 	case '"':
 	  {
-	    op->token = (struct token *) xmalloc (sizeof (struct token));
+	    op->token = XMALLOC (struct token);
 	    init_token (op->token);
 	    op->line_number_at_start = line_number;
 	    for (;;)
@@ -1044,7 +1045,7 @@ read_object (struct object *op, flag_context_ty outer_context)
 	  {
 	    bool symbol;
 
-	    op->token = (struct token *) xmalloc (sizeof (struct token));
+	    op->token = XMALLOC (struct token);
 	    symbol = read_token (op->token, &c);
 	    if (op->token->charcount == 1 && op->token->chars[0] == '.')
 	      {
