@@ -1,12 +1,12 @@
 /* xgettext Scheme backend.
-   Copyright (C) 2004-2006 Free Software Foundation, Inc.
+   Copyright (C) 2004-2007 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <bruno@clisp.org>, 2004-2005.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,13 +14,15 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
+/* Specification.  */
+#include "x-scheme.h"
+
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -32,7 +34,6 @@
 #include "x-scheme.h"
 #include "error.h"
 #include "xalloc.h"
-#include "exit.h"
 #include "hash.h"
 #include "gettext.h"
 
@@ -203,7 +204,7 @@ static inline void
 init_token (struct token *tp)
 {
   tp->allocated = 10;
-  tp->chars = (char *) xmalloc (tp->allocated * sizeof (char));
+  tp->chars = XNMALLOC (tp->allocated, char);
   tp->charcount = 0;
 }
 
@@ -666,7 +667,7 @@ string_of_object (const struct object *op)
   if (!(op->type == t_symbol || op->type == t_string))
     abort ();
   n = op->token->charcount;
-  str = (char *) xmalloc (n + 1);
+  str = XNMALLOC (n + 1, char);
   memcpy (str, op->token->chars, n);
   str[n] = '\0';
   return str;
@@ -794,7 +795,7 @@ read_object (struct object *op, flag_context_ty outer_context)
 		else
 		  {
 		    /* These are the argument positions.  */
-		    if (inner.type == t_string)
+		    if (argparser != NULL && inner.type == t_string)
 		      arglist_parser_remember (argparser, arg,
 					       string_of_object (&inner),
 					       inner_context,
@@ -974,7 +975,7 @@ read_object (struct object *op, flag_context_ty outer_context)
 	      case '{':
 		/* Symbol with multiple escapes: #{...}#  */
 		{
-		  op->token = (struct token *) xmalloc (sizeof (struct token));
+		  op->token = XMALLOC (struct token);
 
 		  init_token (op->token);
 
@@ -1075,7 +1076,7 @@ read_object (struct object *op, flag_context_ty outer_context)
 
 	case '"':
 	  {
-	    op->token = (struct token *) xmalloc (sizeof (struct token));
+	    op->token = XMALLOC (struct token);
 	    init_token (op->token);
 	    op->line_number_at_start = line_number;
 	    for (;;)
@@ -1143,7 +1144,7 @@ read_object (struct object *op, flag_context_ty outer_context)
 	case '5': case '6': case '7': case '8': case '9':
 	case '+': case '-': case '.':
 	  /* Read a number or symbol token.  */
-	  op->token = (struct token *) xmalloc (sizeof (struct token));
+	  op->token = XMALLOC (struct token);
 	  read_token (op->token, c);
 	  if (op->token->charcount == 1 && op->token->chars[0] == '.')
 	    {
@@ -1169,7 +1170,7 @@ read_object (struct object *op, flag_context_ty outer_context)
 	case ':':
 	default:
 	  /* Read a symbol token.  */
-	  op->token = (struct token *) xmalloc (sizeof (struct token));
+	  op->token = XMALLOC (struct token);
 	  read_token (op->token, c);
 	  op->type = t_symbol;
 	  last_non_comment_line = line_number;
